@@ -89,7 +89,7 @@ def merge_datasets(datapaths):
     ds_table=pd.DataFrame(columns=['dataset_name', 'dp', 'probe'])
     for i, (prb, dp) in enumerate(datapaths.items()):
         ds_table.loc[i, 'dataset_name']=op.basename(dp)
-        ds_table.loc[i, 'dp']=dp
+        ds_table.loc[i, 'dp']=str(dp)
         ds_table.loc[i, 'probe']=prb
     ds_table.insert(0, 'dataset_i', ds_table['dataset_name'].argsort()) # sort table by dataset_name alphabetically
     ds_table.sort_values('dataset_i', axis=0, inplace=True)
@@ -203,10 +203,14 @@ def merge_datasets(datapaths):
     return dp_merged, ds_table
 
 def ask_syncchan(ons):
-    chan_len=''.join([f'chan {k} ({len(v)} events).\n' for k,v in ons.items()])
+    chan_len=''.join([f'chan {k} ({len(v)} events).' for k,v in ons.items()])
+    print(f'Data found on sync channels:\n{chan_len}')
+    if len(ons)==1:
+        print('Only one sync channel with data -> using it to synchronize probes.\n')
+        return list(ons.keys())[0]
     syncchan=None
     while (syncchan is None):
-        syncchan=input(f'Data found on sync channels:\n{chan_len}Which channel shall be used to synchronize probes? (q to restart and pick other channel on previous probe) >>> ')
+        syncchan=input('Which channel shall be used to synchronize probes? (q to restart and pick other channel on previous probe) >>> ')
         try:
             syncchan=int(syncchan)
             if syncchan not in ons.keys():
@@ -226,6 +230,7 @@ def ask_syncchan(ons):
 
 
 def get_ds_table(dp):
+    dp = Path(dp)
     ds_table = pd.read_csv(Path(dp, 'datasets_table.csv'), index_col='dataset_i')
     for dp in ds_table['dp']:
         assert op.exists(dp), \
@@ -274,4 +279,6 @@ def get_source_dp_u(dp, u):
         ds_i, u = get_dataset_id(u)
         ds_table=get_ds_table(dp)
         dp=ds_table['dp'][ds_i]
+
+    dp = Path(dp)
     return dp, u
\ No newline at end of file
